//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "preferences.h"
#include <time.h>
#include "opcodes.h"
#include "otherfunctions.h"
#include "ini2.h"
#include "stdlib.h"
#include "stdio.h"
#include <locale.h>
#include "resource.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ELEMENT_COUNT(X) (sizeof(X) / sizeof(X[0]))

bool CPreferences::bWinVerAlreadyDetected = false;

CPreferences::CPreferences(){
	srand((uint32)time(0)); // we need random numbers sometimes
	m_bIsNTBased.Put(false);
	prefs = new Preferences_Struct_DONTUSE;
//	memset(prefs,0,sizeof(Preferences_Struct));
	prefsExt=new Preferences_Ext_Struct;
	memset(prefsExt,0,sizeof(Preferences_Ext_Struct));

	//get application start directory
	char buffer[490];
	::GetModuleFileName(0, buffer, 490);
	LPTSTR pszFileName = _tcsrchr(buffer, '\\') + 1;
	*pszFileName = '\0';
	appdir = nstrdup(buffer);
	CreateUserHash();
	memcpy(&prefs->userhash,&userhash,16);

	// load preferences.dat or set standart values	
	CString fullpath;
	fullpath.Format("%spreferences.dat",appdir);
	FILE* preffile = fopen(fullpath,"rb");

	LoadPreferences();
	if (!preffile){
		SetStandartValues();
		//if (Ask4RegFix(true)) Ask4RegFix(false);
	}
	else{
		fread(prefsExt,sizeof(Preferences_Ext_Struct),1,preffile);
		if (ferror(preffile))
			SetStandartValues();

		// import old pref-files
		if (prefsExt->version<20) {


			if (prefsExt->version>17) {// v0.20b+
				prefsImport20b=new Preferences_Import20b_Struct;
				memset(prefsImport20b,0,sizeof(Preferences_Import20b_Struct));
				fseek(preffile,0,0);
				fread(prefsImport20b,sizeof(Preferences_Import20b_Struct),1,preffile);

				memcpy(&prefs->userhash,&prefsImport20b->userhash,16);
				m_sIncomingDir.Put(prefsImport20b->incomingdir);
				m_sTempDir.Put(prefsImport20b->tempdir);
				m_sNick.Put(prefsImport20b->nick);

				m_nTotalDownloadedBytes.Put(prefsImport20b->totalDownloadedBytes);
				m_nTotalUploadedBytes.Put(prefsImport20b->totalUploadedBytes);

			} else if (prefsExt->version>7) { // v0.20a
				prefsImport20a=new Preferences_Import20a_Struct;
				memset(prefsImport20a,0,sizeof(Preferences_Import20a_Struct));
				fseek(preffile,0,0);
				fread(prefsImport20a,sizeof(Preferences_Import20a_Struct),1,preffile);

				memcpy(&prefs->userhash,&prefsImport20a->userhash,16);
				m_sIncomingDir.Put(prefsImport20a->incomingdir);
				m_sTempDir.Put(prefsImport20a->tempdir);
				m_sNick.Put(prefsImport20a->nick);

				m_nTotalDownloadedBytes.Put(prefsImport20a->totalDownloaded);
				m_nTotalUploadedBytes.Put(prefsImport20a->totalUploaded);

			} else {	//v0.19c-
				prefsImport19c=new Preferences_Import19c_Struct;
				memset(prefsImport19c,0,sizeof(Preferences_Import19c_Struct));

				fseek(preffile,0,0);
				fread(prefsImport19c,sizeof(Preferences_Import19c_Struct),1,preffile);

				if (prefsExt->version<3) {
					CreateUserHash();
					memcpy(&prefs->userhash,&userhash,16);
				} else {memcpy(&prefs->userhash,&prefsImport19c->userhash,16);}
				m_sIncomingDir.Put(prefsImport19c->incomingdir);
				m_sTempDir.Put(prefsImport19c->tempdir);
				m_sNick.Put(prefsImport19c->nick);
			}
 		} else {
			memcpy(&prefs->userhash,&prefsExt->userhash,16);
			prefs->EmuleWindowPlacement=prefsExt->EmuleWindowPlacement;
		}
		fclose(preffile);
		memcpy(&userhash,&prefs->userhash,16);
		m_nSmartIdState.Put(0);
	}

	// shared directories	
	fullpath.Format("%sshareddir.dat",appdir);
	CStdioFile sdirfile;
	if(sdirfile.Open(fullpath,CFile::modeRead))
	{
		CString toadd;
		while (sdirfile.ReadString(toadd))
			shareddir_list.AddHead(toadd);
		sdirfile.Close();
	}

	// start added by InterCeptor (multiple tempdirs) 19.11.02
	// temp directories	
	fullpath.Format("%stempdir.dat",appdir);
	CStdioFile tdirfile;
	if (tdirfile.Open(fullpath,CFile::modeRead)){
		CString toadd;
		while (tdirfile.ReadString(toadd)){
			if(tempdir_list.Find(toadd) == NULL)
			{
				tempdir_list.AddHead(toadd);
				::CreateDirectory(toadd,0);
			}
		}
		tdirfile.Close();
	}
	
	// end added by InterCeptor (multiple tempdirs) 19.11.02

	
	
	//serverlist adresses
	LoadServerlistAddresses() ;

	userhash[5] = 14;
	userhash[14] = 111;

	//Maverick
    m_bUseProxyListenPort.Put(false);
	m_nListenPort.Put(0);

	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
	//CreateDirectory(GetVlcDir(),0);		//Cax2 not needed anymore
}

// Added by Tarod
void CPreferences::LoadServerlistAddresses()
{
	// First we must free list
	adresses_list.RemoveAll() ;

	CString toadd ;
	CString fullpath = CString(appdir) + "adresses.dat" ;
	CStdioFile sdirfile;
	if (sdirfile.Open(fullpath, CFile::modeRead)) {
		adresses_list.RemoveAll() ;
		while (sdirfile.ReadString(toadd)) {
        	// [i_a] allow commented-out entries in the addresses.dat file.
                toadd.TrimLeft();
                if (toadd.GetLength() > 0 && toadd[0] != '#')
			adresses_list.AddHead(toadd) ;
		}
		sdirfile.Close() ;
	}	
}

void CPreferences::SetStandartValues(){
	// CreateUserHash();
	// Saving HashID 
	CString fullpath;
	fullpath.Format("%suserhash.dat", appdir) ;

	FILE* hashfile = fopen(fullpath, "rb") ;
	
	if (hashfile) { 
		if (!fread(userhash, sizeof(userhash), 1, hashfile)) {
            CreateUserHash();
		}
		fclose(hashfile) ;
	}
	// End of Saving HashID 
	memcpy(&prefs->userhash,&userhash,16);

	WINDOWPLACEMENT defaultWPM;
	defaultWPM.length = sizeof(WINDOWPLACEMENT);
	defaultWPM.rcNormalPosition.left=10;defaultWPM.rcNormalPosition.top=10;
	defaultWPM.rcNormalPosition.right=700;defaultWPM.rcNormalPosition.bottom=500;
	defaultWPM.showCmd=0;
	prefs->EmuleWindowPlacement=defaultWPM;

	Save();
}

uint16 CPreferences::GetMaxDownload(){
	//dont be a Lam3r :)
	uint16 nMaxUp = GetMaxUpload();
	uint16 nMaxDown = GetMaxDownloadValue();
	if( nMaxUp < 4 )
		return ( (nMaxUp * 3 < nMaxDown) ? nMaxUp * 3 : nMaxDown);
	return ( ((nMaxUp < 10) && (nMaxUp * 4 < nMaxDown)) ? nMaxUp * 4 : nMaxDown);
}

bool CPreferences::Save(){

	bool error = false;
	CString fullpath;
	fullpath.Format("%spreferences.dat",appdir);

	FILE* preffile = fopen(fullpath,"wb");
	prefsExt->version = PREFFILE_VERSION;
	
	if (preffile){
		prefsExt->version=PREFFILE_VERSION;
		prefsExt->EmuleWindowPlacement=prefs->EmuleWindowPlacement;
		memcpy(&prefsExt->userhash,&prefs->userhash,16);

		error = fwrite(prefsExt,sizeof(Preferences_Ext_Struct),1,preffile);
		fclose(preffile);
	}
	else
		error = true;

	// Saving HashID 	
	fullpath.Format("%suserhash.dat", appdir) ;

	FILE* hashfile = fopen(fullpath, "wb") ;
	
	if (hashfile) {
		error = fwrite(prefs->userhash, sizeof(prefs->userhash), 1, hashfile) ;
		fclose(hashfile) ;
	}
	else
		error = true ;
	// End of Saving HashID

	SavePreferences();
	
	fullpath.Format("%sshareddir.dat",appdir);
	CStdioFile sdirfile;
	if (sdirfile.Open(fullpath,CFile::modeCreate|CFile::modeWrite)){

		for (POSITION pos = shareddir_list.GetHeadPosition();pos != 0;shareddir_list.GetNext(pos)){
			sdirfile.WriteString(shareddir_list.GetAt(pos).GetBuffer());
			sdirfile.Write("\n",1);
		}
		sdirfile.Close();
	}
	else
		error = true;

	// start added by InterCeptor (multiple tempdirs) 19.11.02	
	fullpath.Format("%stempdir.dat",appdir);
	CStdioFile tdirfile;
	if (tdirfile.Open(fullpath,CFile::modeCreate|CFile::modeWrite)){
		for (POSITION pos = tempdir_list.GetHeadPosition();pos != 0;tempdir_list.GetNext(pos)){
			tdirfile.WriteString(tempdir_list.GetAt(pos).GetBuffer());
			tdirfile.Write("\n",1);
		}
		tdirfile.Close();
	}
	else
		error = true;
	// end added by InterCeptor (multiple tempdirs) 19.11.02	
	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
	//CreateDirectory(GetVlcDir(),0);		//Cax2 - not needed anymore with official video player
	return error;
}

void CPreferences::CreateUserHash(){
	for (int i = 0;i != 8; i++){ 
		uint16	random = rand();
		memcpy(&userhash[i*2],&random,2);
	}
	// mark as emule client. that will be need in later version
	userhash[5] = 14;
	userhash[14] = 111;
}

int CPreferences::GetColumnWidth(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnWidths[index];
	case tableUpload:
		return prefs->uploadColumnWidths[index];
	case tableQueue:
		return prefs->queueColumnWidths[index];
	case tableSearch:
		return prefs->searchColumnWidths[index];
	case tableShared:
		return prefs->sharedColumnWidths[index];
	case tableServer:
		return prefs->serverColumnWidths[index];
	case tableIrc:
		return prefs->ircColumnWidths[index];
	}
	return 0;
}

void CPreferences::SetColumnWidth(Table t, int index, int width) {
	switch(t) {
	case tableDownload:
		prefs->downloadColumnWidths[index] = width;
		break;
	case tableUpload:
		prefs->uploadColumnWidths[index] = width;
		break;
	case tableQueue:
		prefs->queueColumnWidths[index] = width;
		break;
	case tableSearch:
		prefs->searchColumnWidths[index] = width;
		break;
	case tableShared:
		prefs->sharedColumnWidths[index] = width;
		break;
	case tableServer:
		prefs->serverColumnWidths[index] = width;
		break;
	case tableIrc:
		prefs->ircColumnWidths[index] = width;
		break;
	}
}

BOOL CPreferences::GetColumnHidden(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnHidden[index];
	case tableUpload:
		return prefs->uploadColumnHidden[index];
	case tableQueue:
		return prefs->queueColumnHidden[index];
	case tableSearch:
		return prefs->searchColumnHidden[index];
	case tableShared:
		return prefs->sharedColumnHidden[index];
	case tableServer:
		return prefs->serverColumnHidden[index];
	case tableIrc:
		return prefs->ircColumnHidden[index];
	}
	return FALSE;
}

void CPreferences::SetColumnHidden(Table t, int index, BOOL bHidden) {
	switch(t) {
	case tableDownload:
		prefs->downloadColumnHidden[index] = bHidden;
		break;
	case tableUpload:
		prefs->uploadColumnHidden[index] = bHidden;
		break;
	case tableQueue:
		prefs->queueColumnHidden[index] = bHidden;
		break;
	case tableSearch:
		prefs->searchColumnHidden[index] = bHidden;
		break;
	case tableShared:
		prefs->sharedColumnHidden[index] = bHidden;
		break;
	case tableServer:
		prefs->serverColumnHidden[index] = bHidden;
		break;
	case tableIrc:
		prefs->ircColumnHidden[index] = bHidden;
		break;
	}
}

int CPreferences::GetColumnOrder(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnOrder[index];
	case tableUpload:
		return prefs->uploadColumnOrder[index];
	case tableQueue:
		return prefs->queueColumnOrder[index];
	case tableSearch:
		return prefs->searchColumnOrder[index];
	case tableShared:
		return prefs->sharedColumnOrder[index];
	case tableServer:
		return prefs->serverColumnOrder[index];
	case tableIrc:
		return prefs->ircColumnOrder[index];
	}
	return 0;
}

void CPreferences::SetColumnOrder(Table t, INT *piOrder) {
	switch(t) {
	case tableDownload:
		memcpy(prefs->downloadColumnOrder, piOrder, sizeof(prefs->downloadColumnOrder));
		break;
	case tableUpload:
		memcpy(prefs->uploadColumnOrder, piOrder, sizeof(prefs->uploadColumnOrder));
		break;
	case tableQueue:
		memcpy(prefs->queueColumnOrder, piOrder, sizeof(prefs->queueColumnOrder));
		break;
	case tableSearch:
		memcpy(prefs->searchColumnOrder, piOrder, sizeof(prefs->searchColumnOrder));
		break;
	case tableShared:
		memcpy(prefs->sharedColumnOrder, piOrder, sizeof(prefs->sharedColumnOrder));
		break;
	case tableServer:
		memcpy(prefs->serverColumnOrder, piOrder, sizeof(prefs->serverColumnOrder));
		break;
	case tableIrc:
		memcpy(prefs->ircColumnOrder, piOrder, sizeof(prefs->ircColumnOrder));
		break;
	}
}

CPreferences::~CPreferences(){
	delete[] appdir;
	delete prefs;
	delete prefsExt;
}

int CPreferences::GetRecommendedMaxConnections() {
	int iRealMax = ::GetMaxConnections();
	if(iRealMax == -1 || iRealMax > 520)
		return 500;

	if(iRealMax < 20)
		return iRealMax;

	if(iRealMax <= 256)
		return iRealMax - 10;

	return iRealMax - 20;
}

void CPreferences::SavePreferences()
{
	EMULE_TRY
	CString buffer;
	CString fullpath; // i_a
	fullpath.Format("%spreferences.ini",appdir);
	
	CIni ini( fullpath, "eMule" );

	//useful!
	ini.WriteString("AppVersion", CURRENT_VERSION_LONG);

	buffer.Format("%s",m_sNick.Get().c_str());
	ini.WriteString("Nick",buffer);

	buffer.Format("%s", m_sIncomingDir.Get().c_str());
	ini.WriteString("IncomingDir",buffer );

	buffer.Format("%s", m_sTempDir.Get().c_str());
	ini.WriteString("TempDir",buffer );

	//buffer.Format("%s", m_sVlcDir.Get().c_str());	//Cax2 we don't save this anymore. m_sVlcDir to be removed in 1b & above
	//ini.WriteString("VLCDir",buffer);
	buffer.Format("%s", m_sVideoPlayer.Get().c_str());	//Cax2 - official mule code.
	ini.WriteString("VideoPlayer",buffer );

	ini.WriteInt("MaxUpload", m_nMaxUpload.Get());
	ini.WriteInt("MaxDownload", m_nMaxDownload.Get());
	ini.WriteInt("MaxConnections", m_nMaxConnections.Get());
	ini.WriteInt("DeadServerAction", m_nDeadServer.Get());
	ini.WriteInt("Port", m_nPort.Get());
	ini.WriteInt("UDPPort", m_nUDPPort.Get());
	ini.WriteInt("MaxSourcesPerFile", m_nMaxSourcePerFile.Get());
	//ini.WriteInt("MaxSourcesPerFileSoft",prefs->maxsourceperfilesoft);
	//ini.WriteInt("MaxSourcesPerFileUDP",prefs->maxsourceperfileudp );
	ini.WriteWORD("Language", m_nLanguageID.Get());
	ini.WriteInt("SeeShare", m_nSeeShares.Get());
	//--- xrmb:partTrafficPrefs ---
	ini.WriteBool("PartTrafficEnabled", m_bUsePartTraffic.Get());
	ini.WriteInt("UploadbarStyle", m_nUpbarPartTraffic.Get());
	ini.WriteInt("UploadbarColor", m_nUpbarColorPartTraffic.Get());
	//--- :xrmb ---
	ini.WriteInt("ToolTipDelay", m_nTooltipDelayTime.Get());
	ini.WriteInt("QueueSize", m_nQueueSize.Get());
	//ini.WriteInt("RemoteQueueLimit", prefs->remotequeuelimit); //remote QR 
	ini.WriteBool("UseRemoteQueueLimit", m_bUseRQlimit.Get());
	ini.WriteInt("StatGraphsInterval", m_nTrafficOMeterInterval.Get());
	ini.WriteInt("StatsInterval", m_nStatsInterval.Get());
	ini.WriteInt("StatsColumnWidth", m_nStatsColumnWidth.Get());
	ini.WriteInt("DetailColumnWidth", m_nDetailColumnWidth.Get());
	ini.WriteString("StatsListExpanded", m_sStatsListExpanded.Get().c_str());
	ini.WriteInt("DownloadCapacity", m_nMaxGraphDownloadRate.Get());
	ini.WriteInt("UploadCapacity", m_nMaxGraphUploadRate.Get());
	ini.WriteInt("DeadServerRetries", m_nDeadServerRetries.Get());
	ini.WriteInt("SplitterbarPosition", m_nSplitterBarPosition.Get());
	ini.WriteInt("VariousStatisticsMaxValue", m_nStatsMax.Get());
	ini.WriteInt("StatsAverageMinutes", m_nStatsAverageMinutes.Get());
	ini.WriteInt("MaxConnectionsPerFiveSeconds", m_nMaxConperFive.Get());
	ini.WriteInt("BadClientBanTime", m_nBadClientBanTime.Get());
	ini.WriteInt("BadClientMinRequestTime", m_nMinRequestTime.Get());
	ini.WriteInt("BadClientMinRequestNum", m_nBadClientBan.Get());
	ini.WriteInt("SLS_RestoreSourcesTime", m_nSLSRestoreSourcesTime.Get());
	ini.WriteInt("SLS_LoadQuantities", m_nSLSLoadQuantities.Get());
	ini.WriteInt("SLS_WhenSourcesOutdated", m_nSLSOutdated.Get());
	ini.WriteInt("SLS_MaxSourcesPerFile",prefs->slsmaxsourcesperfile); // reCDVst
	ini.WriteBool("SLS_Enable", m_bSLSEnable.Get());

	ini.WriteBool("Reconnect", m_bReconnect.Get());
	ini.WriteBool("Scoresystem", m_nScoreSystem.Get());

	ini.WriteBool("UAP", m_bUAP.Get());
	ini.WriteBool("DAP", m_bDAP.Get());
	ini.WriteBool("DisableXS", m_bDisableXS.Get());
	ini.WriteBool("StartDownloadPaused", m_bStartDownloadPaused.Get());
	ini.WriteBool("DownloadPausedOnComplete", m_bDownloadPausedOnComplete.Get());
	ini.WriteBool("RemoveNNS", m_bRemoveNNS.Get());
	
	ini.WriteString("ToolbarSettings", m_sToolbarSettings.Get().c_str()); // Added by reCDVst
	buffer.Format("%s", m_sToolbarBitmap.Get().c_str());
	ini.WriteString("ToolbarBitmap",buffer);
	buffer.Format("%s", m_sToolbarBitmapFolder.Get().c_str());
	ini.WriteString("ToolbarBitmapFolder",buffer);
	ini.WriteInt("ToolbarLabels", m_nToolbarLabels.Get());
	ini.WriteBool("Serverlist", m_bAutoServerList.Get());
	ini.WriteBool("UpdateNotify", m_bUpdateNotify.Get());
	ini.WriteBool("MinimizeToTray", m_bMinToTray.Get());
	ini.WriteBool("AddServersFromConnections", m_bAddServersFromConn.Get());
	ini.WriteBool("Splashscreen", m_bSplashScreen.Get());
	ini.WriteBool("BringToFront", m_bBringToForeground.Get());
	ini.WriteBool("TransferDoubleClick", m_nTransferDoubleclick.Get());
	ini.WriteBool("BeepOnError", m_bBeepOnError.Get());
	ini.WriteBool("ConfirmExit", m_bConfirmExit.Get());
	ini.WriteBool("FilterBadIPs", m_bFilterBadIP.Get());
	ini.WriteBool("Autoconnect", m_bAutoConnect.Get());
	ini.WriteBool("OnlineSignature", m_bOnlineSig.Get());
	ini.WriteBool("StartupMinimized", m_bStartMinimized.Get());
	ini.WriteBool("SafeConnect", m_bSafeConnect.Get());
	ini.WriteBool("ToolbarSpeedMeter", m_bShowToolbarSpeedMeter.Get());
	ini.WriteBool("CloseToTray", m_bCloseToTray.Get());
	ini.WriteBool("DetailsOnClick", m_bDetailsOnClick.Get());
	ini.WriteInt("SrvConTimeout", m_nSrvConTimeout.Get());
	ini.WriteBool("SlowComplete", m_bSlowComplete.Get());
	ini.WriteInt("SlowCompleteBlockSize", m_nSlowCompleteBlockSize.Get());
	ini.WriteBool("ShowRatesOnTitle",prefs->showRatesInTitle);

	// Barry - New properties...
    ini.WriteBool("AutoConnectStaticOnly", m_bAutoConnectStaticOnly.Get());  
    ini.WriteBool("GlobalSearch", prefs->globalsearch);  
    ini.WriteInt ("3DDepth", m_nDepth3D.Get()); 

	// Added by enkeyDEV
	ini.WriteBool("NotifyOnDownload", m_bUseDownloadNotifier.Get());
	ini.WriteBool("NotifyOnDownloadAdd", m_bUseDownloadAddNotifier.Get());
	ini.WriteBool("NotifyOnChat", m_bUseChatNotifier.Get());		  
	ini.WriteBool("NotifyOnLog", m_bUseLogNotifier.Get());
	ini.WriteBool("NotifierUseSound", m_bUseSoundInNotifier.Get());
	ini.WriteBool("NotifierPopEveryChatMessage", m_bNotifierPopsEveryChatMsg.Get());
	ini.WriteBool("NotifyOnImportantError", m_bNotifierImportantError.Get());
	ini.WriteString("NotifierSoundPath", CString(m_sNotifierSoundFilePath.Get().c_str()));

	ini.WriteString("MessageFilter",prefs->messageFilter);
	ini.WriteString("TxtEditor",prefs->TxtEditor);
	ini.WriteString("IRCServer", m_sIRCServer.Get().c_str());
	ini.WriteString("IRCNick", m_sIRCNick.Get().c_str());
	ini.WriteBool("IRCAddTimestamp", m_bIRCAddTimestamp.Get());
	ini.WriteString("IRCFilterName", m_sIRCChanNameFilter.Get().c_str());
	ini.WriteInt("IRCFilterUser", m_nIRCChannelUserFilter.Get());
	ini.WriteBool("IRCUseFilter", m_bIRCUseChanFilter.Get());
	ini.WriteString("IRCPerformString", m_sIRCPerformString.Get().c_str());
	ini.WriteBool("IRCUsePerform", m_bIRCUsePerform.Get());
	ini.WriteBool("IRCListOnConnect", m_bIRCListOnConnect.Get());
	ini.WriteBool("IRCAcceptLinks", m_bIRCAcceptLinks.Get());
	ini.WriteBool("IRCIgnoreInfoMessage", m_bIRCIgnoreInfoMessage.Get());
	ini.WriteBool("IRCIgnoreEmuleProtoInfoMessage", m_bIRCIgnoreEmuleProtoInfoMessage.Get());
	ini.WriteBool("SmartIdCheck", m_bSmartIdCheck.Get());

	ini.WriteBool("UseSortingDefaults", m_bUseSort.Get());
	ini.WriteBool("Use2ndSortSources", m_bUseSrcSortCol2.Get());
	ini.WriteInt("SortServersCode", m_nServerSortCol.Get());	
	ini.WriteInt("SortUploadsCode", m_nUploadSortCol.Get());
	ini.WriteInt("SortQueueCode", m_nQueueSortCol.Get());
	ini.WriteInt("SortSearchCode", m_nSearchSortCol.Get());
	ini.WriteInt("SortIrcCode", m_nIRCSortCol.Get());
	ini.WriteInt("SortFilesCode", m_nFileSortCol.Get());
	ini.WriteInt("SortDownloadsCode", m_nDownloadSortCol.Get());
	ini.WriteInt("1stSortSourcesCode", m_nSrcSortCol1.Get());
	ini.WriteInt("2ndSortSourcesCode", m_nSrcSortCol2.Get());

	// DonGato
	ini.WriteBool("CommunityEnabled", m_bCommunityEnabled.Get());
	ini.WriteString("CommunityString", m_sCommunityString.Get().c_str());
	ini.WriteBool("CommunityNoBan", m_bCommunityNoBan.Get());
	ini.WriteInt("NotificationDisplayTime", m_nNotificationDisplayTime.Get());
	ini.WriteInt("NotificationFontSize", m_nNotificationFontSize.Get());
	ini.WriteInt("Profile", m_nProfile.Get());
	ini.WriteInt("PriorityHigh", m_nPriorityHigh.Get());
	ini.WriteInt("PriorityLow", m_nPriorityLow.Get());
	ini.WriteBool("RestartWaiting", m_bRestartWaiting.Get());
	ini.WriteBool("LogToFile", m_bLogToFile.Get());
	ini.WriteBool("TransferFullChunks", m_bTransferFullChunks.Get());
	ini.WriteBool("BugReport", prefs->BugReport);
	ini.WriteBool("DisableXSUpTo", m_bDisableXSUpTo.Get());
	ini.WriteInt("XSUpTo", m_nXSUpTo.Get());
	ini.WriteString("UsedFont", m_sUsedFont.Get().c_str());
	ini.WriteInt("FontSize", m_nFontSize.Get());
	ini.WriteInt("HashingPriority", m_HashingPriority.Get());
	ini.WriteBool("UploadParts", m_bUploadParts.Get());
	ini.WriteBool("A4AFStringEnabled", m_bA4AFStringEnabled.Get());
	ini.WriteBool("BanMessageEnabled", m_bBanMessageEnabled.Get());
	ini.WriteInt("SearchMethod", m_nSearchMethod.Get());
	ini.WriteBool("AutoTakeED2KLinks", m_bAutoTakeLinks.Get()); 
	ini.WriteInt("FileBufferSizePref", m_nFileBufferSize.Get());
	ini.WriteInt("MainProcessPriority", m_nMainProcessPriority.Get());

	ini.WriteBool("SCHEnabled", m_bSCHEnabled.Get());
	ini.WriteBool("SCHExceptMon", m_bSCHExceptMon.Get());
	ini.WriteBool("SCHExceptTue", m_bSCHExceptTue.Get());
	ini.WriteBool("SCHExceptWed", m_bSCHExceptWed.Get());
	ini.WriteBool("SCHExceptThu", m_bSCHExceptThu.Get());
	ini.WriteBool("SCHExceptFri", m_bSCHExceptFri.Get());
	ini.WriteBool("SCHExceptSat", m_bSCHExceptSat.Get());
	ini.WriteBool("SCHExceptSun", m_bSCHExceptSun.Get());
	ini.WriteInt("SCHShift1",m_nSCHShift1.Get());
	ini.WriteInt("SCHShift2",m_nSCHShift2.Get());
	ini.WriteInt("SCHShift1Up",m_nSCHShift1Upload.Get());
	ini.WriteInt("SCHShift1Down",m_nSCHShift1Download.Get());
	ini.WriteInt("SCHShift1Conn",m_nSCHShift1conn.Get());
	ini.WriteInt("SCHShift15sec",m_nSCHShift15sec.Get());
	ini.WriteInt("SCHShift2Up",m_nSCHShift2Upload.Get());
	ini.WriteInt("SCHShift2Down",m_nSCHShift2Download.Get());
	ini.WriteInt("SCHShift2Conn",m_nSCHShift2conn.Get());
	ini.WriteInt("SCHShift25sec",m_nSCHShift25sec.Get());

	// EpicStruggle
	ini.WriteBool("MultipleInstances", m_bMultiple.Get());

	ini.WriteBool("Verbose", m_bVerbose.Get());
	ini.WriteBool("PreviewPrio", prefs->m_bpreviewprio);
	ini.WriteBool("UpdateQueueList", m_bUpdateQueueList.Get());
	ini.WriteBool("ManualHighPrio", prefs->m_bmanualhighprio);
	//ini.WriteBool("StartNextFile", prefs->m_bstartnextfile); //duplicates downloadpausedoncomplete
	ini.WriteBool("ShowOverhead", m_bShowOverhead.Get());	//Cax2 re-enabled this preference
	ini.WriteBool("VideoPreviewBackupped", m_bBackupPreview.Get());	//Cax2 string kept for compatibility with off client

	ini.SerGet(false, prefs->downloadColumnWidths,
		ELEMENT_COUNT(prefs->downloadColumnWidths), "DownloadColumnWidths");
	ini.SerGet(false, prefs->downloadColumnHidden,
		ELEMENT_COUNT(prefs->downloadColumnHidden), "DownloadColumnHidden");
	ini.SerGet(false, prefs->downloadColumnOrder,
		ELEMENT_COUNT(prefs->downloadColumnOrder), "DownloadColumnOrder");
	ini.SerGet(false, prefs->uploadColumnWidths,
		ELEMENT_COUNT(prefs->uploadColumnWidths), "UploadColumnWidths");
	ini.SerGet(false, prefs->uploadColumnHidden,
		ELEMENT_COUNT(prefs->uploadColumnHidden), "UploadColumnHidden");
	ini.SerGet(false, prefs->uploadColumnOrder,
		ELEMENT_COUNT(prefs->uploadColumnOrder), "UploadColumnOrder");
	ini.SerGet(false, prefs->queueColumnWidths,
		ELEMENT_COUNT(prefs->queueColumnWidths), "QueueColumnWidths");
	ini.SerGet(false, prefs->queueColumnHidden,
		ELEMENT_COUNT(prefs->queueColumnHidden), "QueueColumnHidden");
	ini.SerGet(false, prefs->queueColumnOrder,
		ELEMENT_COUNT(prefs->queueColumnOrder), "QueueColumnOrder");
	ini.SerGet(false, prefs->searchColumnWidths,
		ELEMENT_COUNT(prefs->searchColumnWidths), "SearchColumnWidths");
	ini.SerGet(false, prefs->searchColumnHidden,
		ELEMENT_COUNT(prefs->searchColumnHidden), "SearchColumnHidden");
	ini.SerGet(false, prefs->searchColumnOrder,
		ELEMENT_COUNT(prefs->searchColumnOrder), "SearchColumnOrder");
	ini.SerGet(false, prefs->sharedColumnWidths,
		ELEMENT_COUNT(prefs->sharedColumnWidths), "SharedColumnWidths");
	ini.SerGet(false, prefs->sharedColumnHidden,
		ELEMENT_COUNT(prefs->sharedColumnHidden), "SharedColumnHidden");
	ini.SerGet(false, prefs->sharedColumnOrder,
		ELEMENT_COUNT(prefs->sharedColumnOrder), "SharedColumnOrder");
	ini.SerGet(false, prefs->serverColumnWidths,
		ELEMENT_COUNT(prefs->serverColumnWidths), "ServerColumnWidths");
	ini.SerGet(false, prefs->serverColumnHidden,
		ELEMENT_COUNT(prefs->serverColumnHidden), "ServerColumnHidden");
	ini.SerGet(false, prefs->serverColumnOrder,
		ELEMENT_COUNT(prefs->serverColumnOrder), "ServerColumnOrder");
	ini.SerGet(false, prefs->ircColumnWidths,
		ELEMENT_COUNT(prefs->ircColumnWidths), "IrcColumnWidths");
	ini.SerGet(false, prefs->ircColumnHidden,
		ELEMENT_COUNT(prefs->ircColumnHidden), "IrcColumnHidden");
	ini.SerGet(false, prefs->ircColumnOrder,
		ELEMENT_COUNT(prefs->ircColumnOrder), "IrcColumnOrder");

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	ini.WriteInt("TableSortItemDownload", prefs->tableSortItemDownload);
	ini.WriteInt("TableSortItemUpload", prefs->tableSortItemUpload);
	ini.WriteInt("TableSortItemQueue", prefs->tableSortItemQueue);
	ini.WriteInt("TableSortItemSearch", prefs->tableSortItemSearch);
	ini.WriteInt("TableSortItemShared", prefs->tableSortItemShared);
	ini.WriteInt("TableSortItemServer", prefs->tableSortItemServer);
	ini.WriteBool("TableSortAscendingDownload", prefs->tableSortAscendingDownload);
	ini.WriteBool("TableSortAscendingUpload", prefs->tableSortAscendingUpload);
	ini.WriteBool("TableSortAscendingQueue", prefs->tableSortAscendingQueue);
	ini.WriteBool("TableSortAscendingSearch", prefs->tableSortAscendingSearch);
	ini.WriteBool("TableSortAscendingShared", prefs->tableSortAscendingShared);
	ini.WriteBool("TableSortAscendingServer", prefs->tableSortAscendingServer);

	CString buffer2;
	for (int i=0;i<15;i++) {
		buffer.Format("%I64Lu",GetStatsColor(i));
		buffer2.Format("StatColor%i",i);
		ini.WriteString(buffer2,buffer);
	}
	ini.WriteInt("ConnectionsGraphRatio", m_nGraphRatio.Get()); // propstats Cax2

	// Added By Maverick
	ini.WriteBool("MlDonkeyCM", m_bMlDonkeyCM.Get());

	// Added by Tarod -saving HASH of A4AF auto file
	char hex_hash[33];
	hex_hash[0] = '\0';
	for (uint16 i = 0; i < 16; i++) // hmm I wonder if there is a standart function for this
		sprintf(hex_hash, "%s%02X", hex_hash, prefs->A4AF_file_hash[i]);
	ini.WriteString("A4AFAutoFileHash", CString(hex_hash));	

	// Maverick
	ini.WriteBool("EnablePassword",prefs->proxy.EnablePassword,"Proxy");
	ini.WriteBool("EnableProxy",prefs->proxy.UseProxy,"Proxy");
	ini.WriteString("Name",prefs->proxy.name,"Proxy");
	ini.WriteString("Password",prefs->proxy.password,"Proxy");
	ini.WriteString("User",prefs->proxy.user,"Proxy");
	ini.WriteInt("Port",prefs->proxy.port,"Proxy");
	ini.WriteInt("Type",prefs->proxy.type,"Proxy");

	buffer.Format("%I64Lu", m_nTotalDownloadedBytes.Get());
	ini.WriteString("TotalDownloadedBytes",buffer ,"Statistics");

	buffer.Format("%I64Lu", m_nTotalUploadedBytes.Get());
	ini.WriteString("TotalUploadedBytes",buffer ,"Statistics");

	// Web Server
	ini.WriteString("Password", CString(m_sWebPassword.Get().c_str()), "WebServer");	
	ini.WriteInt("Port", m_nWebPort.Get(), "WebServer");
	ini.WriteBool("Enabled", m_bWebEnabled.Get(), "WebServer");
	ini.WriteBool("UseGzip", m_bWebUseGzip.Get(), "WebServer");
	ini.WriteInt("PageRefreshTime", m_nWebPageRefresh.Get(), "WebServer");

	//////////////////////////////////////////////////////////////////
	// LANCAST (moosetea) - Save the settings
	
	char t_strLanCastIP[17];
	sprintf(t_strLanCastIP,"%i.%i.%i.%i",(uint8)m_LancastIP.Get(),(uint8)(m_LancastIP.Get()>>8),(uint8)(m_LancastIP.Get()>>16),(uint8)(m_LancastIP.Get()>>24));

	char t_strLanCastSubnet[17];
	sprintf(t_strLanCastSubnet,"%i.%i.%i.%i",(uint8)m_LancastSubnet.Get(),(uint8)(m_LancastSubnet.Get()>>8),(uint8)(m_LancastSubnet.Get()>>16),(uint8)(m_LancastSubnet.Get()>>24));
				
	ini.WriteBool("Enabled",m_LancastEnabled.Get(),"Lancast");
	ini.WriteBool("UseDefaultIP",m_UseDefaultIP.Get(),"Lancast");
	ini.WriteString("IPAddress",t_strLanCastIP,"Lancast");
	ini.WriteString("Subnet",t_strLanCastSubnet,"Lancast");
	ini.WriteInt("Port",m_LancastPort.Get(),"Lancast");

	//////////////////////////////////////////////////////////////////

	//START enkeyDEV(kei-kun) -Messaging preferences-
	ini.WriteInt("AcceptMessageFrom",m_nAcceptMessagesFrom.Get());	
	ini.WriteString("AwayStateMessage", CString(m_sAwayStateMessage.Get().c_str()));
	//END enkeyDEV(kei-kun) -Messaging preferences-
	EMULE_CATCH
}

DWORD CPreferences::DefaultStatsColor(int index){
	switch(index)
	{
		case 0 : return RGB(0,0,64);
		case 1 : return RGB(192,192,255);
		case 2 : return RGB(128, 255, 128);
		case 3 : return RGB(0, 210, 0);
		case 4 : return RGB(0, 128, 0);
		case 5 : return RGB(255, 128, 128);
		case 6 : return RGB(200, 0, 0);
		case 7 : return RGB(140, 0, 0);
		case 8 : return RGB(150, 150, 255);
		case 9 : return RGB(192,   0, 192);
		case 10 : return RGB(255, 192, 255);
		case 11 : return RGB(255, 255, 128);
		default : return RGB(255, 255, 255);
	}
}

void CPreferences::LoadPreferences(){
//	CSingleLock PrefLock(&m_Pref_Mutex,TRUE);
	char buffer[200];

	//--- Quick hack to add version tag to preferences.ini-file and solve the issue with the FlatStatusBar tag...
	CString strFileName;
	strFileName.Format("%spreferences.ini", appdir);
	CIni* pIni = new CIni(strFileName, "eMule");
	CString strCurrVersion, strPrefsVersion;
	strCurrVersion = CURRENT_VERSION_LONG;
	strPrefsVersion = pIni->GetString("AppVersion");
	delete pIni;
	CFileFind findFileName;

	if ((strCurrVersion != strPrefsVersion) && findFileName.FindFile(strFileName))
	{
		CFile file;
		CFileFind findNewName;
		CString strNewName;
		//Cax2 we can add the code to find our version's setting, if the ini file is from a different version...
		strNewName.Format("%spreferences[%s].ini", appdir,strPrefsVersion);	
		if (findNewName.FindFile(strNewName))
			file.Remove(strNewName);
		file.Rename(strFileName, strNewName);
		strFileName = strNewName;
	}

	CIni ini(strFileName, "eMule");
	//--- end Ozon :)

	CString strDefNick;	// random nick [FoRcHa]
	srand((unsigned)time(NULL));
	for(int i = 0; i < 6; i++)
		buffer[i] = rand() % ('z' - 'a') + 'a';
	buffer[i] = '\0';
	strDefNick.Format("%s [eMule Plus v%s]", buffer, CURRENT_VERSION_LONG);
	m_sNick.Put(ini.GetString("Nick",strDefNick).GetBuffer(0));
	
	sprintf(buffer,"%sIncoming",appdir);
	m_sIncomingDir.Put(ini.GetString("IncomingDir",buffer).GetBuffer(0));

	sprintf(buffer,"%sTemp",appdir);
	m_sTempDir.Put(ini.GetString("TempDir",buffer).GetBuffer(0));

	sprintf(buffer,"%sVLC",appdir);
	m_sVlcDir.Put(ini.GetString("VLCDir",buffer).GetBuffer(0));
	sprintf(buffer,"%s\\vlc.exe",GetVlcDir());		//Cax2 - for backward compatibility
	m_sVideoPlayer.Put(ini.GetString("VideoPlayer",buffer).GetBuffer(0));		//Cax2 - replace buffer with "" in later (non b-compatible) versions
	m_nMaxGraphUploadRate.Put(ini.GetInt("UploadCapacity",16));
	m_nMaxGraphDownloadRate.Put(ini.GetInt("DownloadCapacity",96));
	m_nMaxUpload.Put(ini.GetInt("MaxUpload",UNLIMITED));		//Cax2 new limits enforced on old settings
	if (m_nMaxUpload.Get()>m_nMaxGraphUploadRate.Get())
		m_nMaxUpload.Put(m_nMaxGraphUploadRate.Get());
	m_nMaxDownload.Put(ini.GetInt("MaxDownload",UNLIMITED));
	if (m_nMaxDownload.Get()>m_nMaxGraphDownloadRate.Get())
		m_nMaxDownload.Put(m_nMaxGraphDownloadRate.Get());
	m_nMaxConnections.Put(ini.GetInt("MaxConnections",GetRecommendedMaxConnections()));
	m_nDeadServer.Put(ini.GetInt("DeadServerAction",2));
	m_nPort.Put(ini.GetInt("Port",4662));
	m_nUDPPort.Put(ini.GetInt("UDPPort",m_nPort.Get() + 10));
	m_nMaxSourcePerFile.Put(ini.GetInt("MaxSourcesPerFile",400));
	//prefs->maxsourceperfilesoft=ini.GetInt("MaxSourcesPerFileSoft",360);
	//prefs->maxsourceperfileudp=ini.GetInt("MaxSourcesPerFileUDP",100);
	m_nQueueSize.Put(ini.GetInt("QueueSize", MIN_QUEUE*2));
	if(m_nQueueSize.Get() > MAX_QUEUE)
		m_nQueueSize.Put(MAX_QUEUE);
	else if(m_nQueueSize.Get() < MIN_QUEUE)
		m_nQueueSize.Put(MIN_QUEUE);
	m_nRQlimit.Put((m_nQueueSize.Get() >= 2000)? m_nQueueSize.Get() : 2000); //Cax2 - remote queue limit linked to queue size, but no lower than 2000
	m_bUseRQlimit.Put(ini.GetBool("UseRemoteQueueLimit"));
	m_nLanguageID.Put(ini.GetWORD("Language",0));
	m_nSeeShares.Put((enSeeShare)ini.GetInt("SeeShare",SEE_SHARE_EVERYBODY));
	//--- xrmb:partTrafficPrefs ---
	m_bUsePartTraffic.Put(ini.GetBool("PartTrafficEnabled",false));
	m_nUpbarPartTraffic.Put(ini.GetInt("UploadbarStyle", 2));
	m_nUpbarColorPartTraffic.Put(ini.GetInt("UploadbarColor", 0));
	//--- :xrmb ---
	m_nTooltipDelayTime.Put(ini.GetInt("ToolTipDelay",3));
	m_nTrafficOMeterInterval.Put(ini.GetInt("StatGraphsInterval",3));
	m_nStatsInterval.Put(ini.GetInt("statsInterval",5));
	m_nStatsColumnWidth.Put(ini.GetInt("StatsColumnWidth", 150));
	m_nDetailColumnWidth.Put(ini.GetInt("DetailColumnWidth", 150));
	
	m_sStatsListExpanded.Put(ini.GetString("StatsListExpanded", "").GetBuffer(0)); // missing entrys get expanded ("" = all expanded)

	m_nDeadServerRetries.Put(ini.GetInt("DeadServerRetries",1));
	m_nSplitterBarPosition.Put(ini.GetInt("SplitterbarPosition",75));
	m_nStatsMax.Put(ini.GetInt("VariousStatisticsMaxValue",100));
	m_nStatsAverageMinutes.Put(ini.GetInt("StatsAverageMinutes",5));
	m_nMaxConperFive.Put(ini.GetInt("MaxConnectionsPerFiveSeconds",GetDefaultMaxConperFive()));
	prefs->showRatesInTitle=ini.GetBool("ShowRatesOnTitle",false);

	// DonGato
	int nMax = (LOBYTE(GetWindowsVersion()) == 0x04)?100:200;
	if(m_nMaxConperFive.Get() > nMax) {
		m_nMaxConperFive.Put(nMax);
	} else {
		if(m_nMaxConperFive.Get() < 5)
			m_nMaxConperFive.Put(5);
	}

	m_nBadClientBanTime.Put(ini.GetInt("BadClientBanTime",3600000));
	if(m_nBadClientBanTime.Get() > 18000000)
		m_nBadClientBanTime.Put(18000000);
	
	m_nMinRequestTime.Put(ini.GetInt("BadClientMinRequestTime",590000));
	if(m_nMinRequestTime.Get() > 590000)
		m_nMinRequestTime.Put(590000);
	
	m_nBadClientBan.Put(ini.GetInt("BadClientMinRequestNum",3));
	if(m_nBadClientBan.Get() < 3)
		m_nBadClientBan.Put(3);

	// Original by reCDVst updated by [DonGato]
	m_nSLSRestoreSourcesTime.Put(ini.GetInt("SLS_RestoreSourcesTime",7200000));
	if(m_nSLSRestoreSourcesTime.Get() < 3600000)
		m_nSLSRestoreSourcesTime.Put(3600000);
	
	m_nSLSLoadQuantities.Put(ini.GetInt("SLS_LoadQuantities",20));
	if(m_nSLSLoadQuantities.Get() > 100)
		m_nSLSLoadQuantities.Put(100);

	m_nSLSOutdated.Put(ini.GetInt("SLS_WhenSourcesOutdated",2));
	if(m_nSLSOutdated.Get() > 10)
		m_nSLSOutdated.Put(10);
	
	m_bSLSEnable.Put(ini.GetBool("SLS_Enable",false));

	m_bReconnect.Put(ini.GetBool("Reconnect",true));
	m_nScoreSystem.Put(ini.GetBool("Scoresystem",true));
	
	m_bUAP.Put(ini.GetBool("UAP", true));
	m_bDAP.Put(ini.GetBool("DAP", true));
	m_bDisableXS.Put(ini.GetBool("DisableXS", false));
	m_bStartDownloadPaused.Put(ini.GetBool("StartDownloadPaused", false));
	m_bDownloadPausedOnComplete.Put(ini.GetBool("DownloadPausedOnComplete", false));
	m_bRemoveNNS.Put(ini.GetBool("RemoveNNS", false));
	
	m_sToolbarSettings.Put(ini.GetString("ToolbarSettings", strDefaultToolbar).GetBuffer(0));
	m_sToolbarBitmap.Put(ini.GetString("ToolbarBitmap", "").GetBuffer(0)); // Added by reCDVst
	m_sToolbarBitmapFolder.Put(ini.GetString("ToolbarBitmapFolder", "").GetBuffer(0)); // Added by reCDVst
	m_nToolbarLabels.Put(ini.GetInt("ToolbarLabels",1));
	m_bAutoServerList.Put(ini.GetBool("Serverlist",false));
	m_bUpdateNotify.Put(ini.GetBool("UpdateNotify",true));
	m_bMinToTray.Put(ini.GetBool("MinimizeToTray",true));
	m_bAddServersFromConn.Put(ini.GetBool("AddServersFromConnections",true));
	m_bSplashScreen.Put(ini.GetBool("Splashscreen",true));
	m_bBringToForeground.Put(ini.GetBool("BringToFront",true));
	m_nTransferDoubleclick.Put(ini.GetBool("TransferDoubleClick",true));
	m_bBeepOnError.Put(ini.GetBool("BeepOnError",true));
	m_bConfirmExit.Put(ini.GetBool("ConfirmExit",true));
	m_bFilterBadIP.Put(ini.GetBool("FilterBadIPs",true));
	m_bAutoConnect.Put(ini.GetBool("Autoconnect",false));
	m_bOnlineSig.Put(ini.GetBool("OnlineSignature",false));
	m_bStartMinimized.Put(ini.GetBool("StartupMinimized",false));
	m_bSafeConnect.Put(ini.GetBool("SafeConnect",true));
	m_bShowToolbarSpeedMeter.Put(ini.GetBool("ToolbarSpeedMeter", false));
	m_bCloseToTray.Put(ini.GetBool("CloseToTray",false));	
	m_bDetailsOnClick.Put(ini.GetBool("DetailsOnClick",false));	

	m_nSrvConTimeout.Put(ini.GetInt("SrvConTimeout",20000));		//Cax2 - bugfix for time out - ticks are in milliseconds!!!!!
	if(m_nSrvConTimeout.Get() <10000)
		m_nSrvConTimeout.Put(10000);
	else if(m_nSrvConTimeout.Get() > 60000)
		m_nSrvConTimeout.Put(60000);
	
	m_bSlowComplete.Put(ini.GetBool("SlowComplete",false));	
	m_nSlowCompleteBlockSize.Put(ini.GetInt("SlowCompleteBlockSize",32));
	if(m_nSlowCompleteBlockSize.Get() > 1024)
		m_nSlowCompleteBlockSize.Put(1024);

	// Barry - New properties...
	m_bAutoConnectStaticOnly.Put(ini.GetBool("AutoConnectStaticOnly",false)); 
	prefs->globalsearch = ini.GetBool("GlobalSearch",true); 
	m_nDepth3D.Put(ini.GetInt("3DDepth", 0));

	// as a temporary converter for previous versions
	if (strPrefsVersion < "0.25a") // before 0.25a
		if (ini.GetBool("FlatStatusBar",false))
			m_nDepth3D.Put(0);
		else 
			m_nDepth3D.Put(5);

	// Added by enkeyDEV
	m_bUseDownloadAddNotifier.Put(ini.GetBool("NotifyOnDownloadAdd",false));
	m_bUseDownloadNotifier.Put(ini.GetBool("NotifyOnDownload",false));
	m_bUseChatNotifier.Put(ini.GetBool("NotifyOnChat",false));
	m_bUseLogNotifier.Put(ini.GetBool("NotifyOnLog",false));
	m_bUseSoundInNotifier.Put(ini.GetBool("NotifierUseSound",false));
	m_bNotifierPopsEveryChatMsg.Put(ini.GetBool("NotifierPopEveryChatMessage",false));
	m_bNotifierImportantError.Put(ini.GetBool("NotifyOnImportantError",false));
	m_sNotifierSoundFilePath.Put(ini.GetString("NotifierSoundPath","").GetBuffer(0));

	m_sIRCServer.Put(ini.GetString("IRCServer","irc.liquidirc.com").GetBuffer(0));
	m_sIRCNick.Put(ini.GetString("IRCNick","eMule Plus").GetBuffer(0));
	m_bIRCAddTimestamp.Put(ini.GetBool("IRCAddTimestamp",true));
	m_sIRCChanNameFilter.Put(ini.GetString("IRCFilterName", "").GetBuffer(0));
	m_bIRCUseChanFilter.Put(ini.GetBool("IRCUseFilter", false));
	m_nIRCChannelUserFilter.Put(ini.GetInt("IRCFilterUser", 0));
	m_sIRCPerformString.Put(ini.GetString("IRCPerformString", "/join #emule").GetBuffer(0));
	m_bIRCUsePerform.Put(ini.GetBool("IRCUsePerform", false));
	m_bIRCListOnConnect.Put(ini.GetBool("IRCListOnConnect", true));
	m_bIRCAcceptLinks.Put(ini.GetBool("IRCAcceptLinks", false));
	m_bIRCIgnoreInfoMessage.Put(ini.GetBool("IRCIgnoreInfoMessage", false));
	m_bIRCIgnoreEmuleProtoInfoMessage.Put(ini.GetBool("IRCIgnoreEmuleProtoInfoMessage", true));
	m_bSmartIdCheck.Put(ini.GetBool("SmartIdCheck",true));
	m_bVerbose.Put(ini.GetBool("Verbose",false));
	prefs->m_bpreviewprio=ini.GetBool("PreviewPrio",false);
	m_bUpdateQueueList.Put(ini.GetBool("UpdateQueueList",false));
	prefs->m_bmanualhighprio=ini.GetBool("ManualHighPrio",false);
	m_bShowOverhead.Put(ini.GetBool("ShowOverhead",true));
	m_bBackupPreview.Put(ini.GetBool("VideoPreviewBackupped",false));	//Cax2 string kept for compatibility with off client

	sprintf(prefs->TxtEditor,"%s",ini.GetString("TxtEditor","notepad.exe"));
	prefs->messageFilter=ini.GetString("MessageFilter","Your client has an infinite queue");

	m_bUseSort.Put(ini.GetBool("UseSortingDefaults",true));
	m_bUseSrcSortCol2.Put(ini.GetBool("Use2ndSortSources",false));
	m_nServerSortCol.Put(ini.GetInt("SortServersCode",0));
	m_nUploadSortCol.Put(ini.GetInt("SortUploadsCode",12));
	m_nQueueSortCol.Put(ini.GetInt("SortQueueCode",105));
	m_nSearchSortCol.Put(ini.GetInt("SortSearchCode",12));
	m_nIRCSortCol.Put(ini.GetInt("SortIrcCode",11));
	m_nFileSortCol.Put(ini.GetInt("SortFilesCode",0));
	m_nDownloadSortCol.Put(ini.GetInt("SortDownloadsCode",8));
	m_nSrcSortCol1.Put(ini.GetInt("1stSortSourcesCode",0));
	m_nSrcSortCol2.Put(ini.GetInt("2ndSortSourcesCode",255));

	// DonGato
	m_bCommunityEnabled.Put(ini.GetBool("CommunityEnabled", false));
	m_sCommunityString.Put(ini.GetString("CommunityString","").GetBuffer(0));
	m_bCommunityNoBan.Put(ini.GetBool("CommunityNoBan", false));
	m_nNotificationDisplayTime.Put(ini.GetInt("NotificationDisplayTime",4000));
	m_nNotificationFontSize.Put(ini.GetInt("NotificationFontSize",70));
	m_nProfile.Put(ini.GetInt("Profile",0));
	m_nPriorityHigh.Put(ini.GetInt("PriorityHigh",100));
	m_nPriorityLow.Put(ini.GetInt("PriorityLow",300));
	if(m_nPriorityLow.Get() != 0){
		if(m_nPriorityLow.Get() < m_nPriorityHigh.Get())
			m_nPriorityLow.Put(m_nPriorityHigh.Get() * 2);
	} else {
		m_nPriorityLow.Put(300);
		m_nPriorityHigh.Put(100);
	}
	m_bRestartWaiting.Put(ini.GetBool("RestartWaiting",true));
	m_bLogToFile.Put(ini.GetBool("LogToFile",false));
	m_bTransferFullChunks.Put(ini.GetBool("TransferFullChunks",true));
	prefs->BugReport = ini.GetBool("BugReport",false);
	m_bDisableXSUpTo.Put(ini.GetBool("DisableXSUpTo",false));
	m_nXSUpTo.Put(ini.GetInt("XSUpTo",GetMaxSourcePerFileSoft()));
	if(m_nXSUpTo.Get() > GetMaxSourcePerFileSoft())
		m_nXSUpTo.Put(GetMaxSourcePerFileSoft());
	m_sUsedFont.Put(ini.GetString("UsedFont", "MS Sans Serif").GetBuffer(0));
	m_nFontSize.Put(ini.GetInt("FontSize",80));
	m_HashingPriority.Put((enHashPriority)ini.GetInt("HashingPriority",2));
	m_bUploadParts.Put(ini.GetBool("UploadParts",false));
	m_bA4AFStringEnabled.Put(ini.GetBool("A4AFStringEnabled", false));
	m_bBanMessageEnabled.Put(ini.GetBool("BanMessageEnabled", false));
	m_nSearchMethod.Put(ini.GetInt("SearchMethod",0));
	m_bBanMessageEnabled.Put(ini.GetBool("BanMessageEnabled", false));
	m_bAutoTakeLinks.Put(ini.GetBool("AutoTakeED2KLinks",false));
	m_nFileBufferSize.Put(ini.GetInt("FileBufferSizePref",16));
	m_nMainProcessPriority.Put(ini.GetInt("MainProcessPriority",1));

	m_bSCHEnabled.Put(ini.GetBool("SCHEnabled", false));
	m_bSCHExceptMon.Put(ini.GetBool("SCHExceptMon", false));
	m_bSCHExceptTue.Put(ini.GetBool("SCHExceptTue", false));
	m_bSCHExceptWed.Put(ini.GetBool("SCHExceptWed", false));
	m_bSCHExceptThu.Put(ini.GetBool("SCHExceptThu", false));
	m_bSCHExceptFri.Put(ini.GetBool("SCHExceptFri", false));
	m_bSCHExceptSat.Put(ini.GetBool("SCHExceptSat", true));
	m_bSCHExceptSun.Put(ini.GetBool("SCHExceptSun", true));
	m_nSCHShift1.Put(ini.GetInt("SCHShift1",64800));
	m_nSCHShift2.Put(ini.GetInt("SCHShift2",21600));
	m_nSCHShift1Upload.Put(ini.GetInt("SCHShift1Up",UNLIMITED));
	m_nSCHShift1Download.Put(ini.GetInt("SCHShift1Down",UNLIMITED));
	m_nSCHShift1conn.Put(ini.GetInt("SCHShift1conn",512));
	m_nSCHShift15sec.Put(ini.GetInt("SCHShift15sec",40));
	m_nSCHShift2Upload.Put(ini.GetInt("SCHShift2Up",UNLIMITED));
	m_nSCHShift2Download.Put(ini.GetInt("SCHShift2Down",UNLIMITED));
	m_nSCHShift2conn.Put(ini.GetInt("SCHShift2conn",256));
	m_nSCHShift25sec.Put(ini.GetInt("SCHShift25sec",20));

	m_bMultiple.Put(ini.GetBool("MultipleInstances",false));

	ini.SerGet(true, prefs->downloadColumnWidths,
		ELEMENT_COUNT(prefs->downloadColumnWidths), "DownloadColumnWidths");
	ini.SerGet(true, prefs->downloadColumnHidden,
		ELEMENT_COUNT(prefs->downloadColumnHidden), "DownloadColumnHidden");
	ini.SerGet(true, prefs->downloadColumnOrder,
		ELEMENT_COUNT(prefs->downloadColumnOrder), "DownloadColumnOrder");
	ini.SerGet(true, prefs->uploadColumnWidths,
		ELEMENT_COUNT(prefs->uploadColumnWidths), "UploadColumnWidths");
	ini.SerGet(true, prefs->uploadColumnHidden,
		ELEMENT_COUNT(prefs->uploadColumnHidden), "UploadColumnHidden");
	ini.SerGet(true, prefs->uploadColumnOrder,
		ELEMENT_COUNT(prefs->uploadColumnOrder), "UploadColumnOrder");
	ini.SerGet(true, prefs->queueColumnWidths,
		ELEMENT_COUNT(prefs->queueColumnWidths), "QueueColumnWidths");
	ini.SerGet(true, prefs->queueColumnHidden,
		ELEMENT_COUNT(prefs->queueColumnHidden), "QueueColumnHidden");
	ini.SerGet(true, prefs->queueColumnOrder,
		ELEMENT_COUNT(prefs->queueColumnOrder), "QueueColumnOrder");
	ini.SerGet(true, prefs->searchColumnWidths,
		ELEMENT_COUNT(prefs->searchColumnWidths), "SearchColumnWidths");
	ini.SerGet(true, prefs->searchColumnHidden,
		ELEMENT_COUNT(prefs->searchColumnHidden), "SearchColumnHidden");
	ini.SerGet(true, prefs->searchColumnOrder,
		ELEMENT_COUNT(prefs->searchColumnOrder), "SearchColumnOrder");
	ini.SerGet(true, prefs->sharedColumnWidths,
		ELEMENT_COUNT(prefs->sharedColumnWidths), "SharedColumnWidths");
	ini.SerGet(true, prefs->sharedColumnHidden,
		ELEMENT_COUNT(prefs->sharedColumnHidden), "SharedColumnHidden");
	ini.SerGet(true, prefs->sharedColumnOrder,
		ELEMENT_COUNT(prefs->sharedColumnOrder), "SharedColumnOrder");
	ini.SerGet(true, prefs->serverColumnWidths,
		ELEMENT_COUNT(prefs->serverColumnWidths), "ServerColumnWidths");
	ini.SerGet(true, prefs->serverColumnHidden,
		ELEMENT_COUNT(prefs->serverColumnHidden), "ServerColumnHidden");
	ini.SerGet(true, prefs->serverColumnOrder,
		ELEMENT_COUNT(prefs->serverColumnOrder), "ServerColumnOrder");
	ini.SerGet(true, prefs->ircColumnWidths,
		ELEMENT_COUNT(prefs->ircColumnWidths), "IrcColumnWidths");
	ini.SerGet(true, prefs->ircColumnHidden,
		ELEMENT_COUNT(prefs->ircColumnHidden), "IrcColumnHidden");
	ini.SerGet(true, prefs->ircColumnOrder,
		ELEMENT_COUNT(prefs->ircColumnOrder), "IrcColumnOrder");

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	prefs->tableSortItemDownload = ini.GetInt("TableSortItemDownload", 0);
	prefs->tableSortItemUpload = ini.GetInt("TableSortItemUpload", 0);
	prefs->tableSortItemQueue = ini.GetInt("TableSortItemQueue", 0);
	prefs->tableSortItemSearch = ini.GetInt("TableSortItemSearch", 0);
	prefs->tableSortItemShared = ini.GetInt("TableSortItemShared", 0);
	prefs->tableSortItemServer = ini.GetInt("TableSortItemServer", 0);
	prefs->tableSortAscendingDownload = ini.GetBool("TableSortAscendingDownload", true);
	prefs->tableSortAscendingUpload = ini.GetBool("TableSortAscendingUpload", true);
	prefs->tableSortAscendingQueue = ini.GetBool("TableSortAscendingQueue", true);
	prefs->tableSortAscendingSearch = ini.GetBool("TableSortAscendingSearch", true);
	prefs->tableSortAscendingShared = ini.GetBool("TableSortAscendingShared", true);
	prefs->tableSortAscendingServer = ini.GetBool("TableSortAscendingServer", true);

	if (m_nStatsAverageMinutes.Get() < 1)
		m_nStatsAverageMinutes.Put(5);

	CString buffer2;
	for (int i=0;i<15;i++) {
		buffer2.Format("StatColor%i",i);
		sprintf(buffer,ini.GetString(buffer2,"-1"));	//we can now have black as background
		m_nStatColors[i].Put(_atoi64(buffer));
		if (m_nStatColors[i].Get() == -1) 
			m_nStatColors[i].Put(DefaultStatsColor(i));
	}
	m_nGraphRatio.Put(ini.GetInt("ConnectionsGraphRatio",1));

	m_bMlDonkeyCM.Put(ini.GetBool("MlDonkeyCM",true));

	// Added by Tarod -to load hash of A4AF auto file
	char hex_byte[3];
	int byte;
	CString A4AF_hash = ini.GetString("A4AFAutoFileHash", "00000000000000000000000000000000");
	if (A4AF_hash.GetLength() != 32) A4AF_hash = "00000000000000000000000000000000";
	hex_byte[2] = '\0';
	for (int i = 0, j = 0; i < 16; i++, j += 2) {
		hex_byte[0] = A4AF_hash.GetAt(j);
		hex_byte[1] = A4AF_hash.GetAt((j + 1));
		sscanf(hex_byte, "%02X", &byte);
		prefs->A4AF_file_hash[i] = (uchar)byte;
	}

	sprintf(buffer,ini.GetString("TotalDownloadedBytes",0,"Statistics"));
	m_nTotalDownloadedBytes.Put(_atoi64(buffer));

	sprintf(buffer,ini.GetString("TotalUploadedBytes",0,"Statistics"));
	m_nTotalUploadedBytes.Put(_atoi64(buffer));
	// Maverick
	prefs->proxy.EnablePassword = ini.GetBool("EnablePassword",false,"Proxy");
	prefs->proxy.UseProxy = ini.GetBool("EnableProxy",false,"Proxy");
	sprintf(buffer,"");
	sprintf(prefs->proxy.name,"%s",ini.GetString("Name",buffer,"Proxy"));
	sprintf(prefs->proxy.password,"%s",ini.GetString("Password",buffer,"Proxy"));
	sprintf(prefs->proxy.user,"%s",ini.GetString("User",buffer,"Proxy"));
	prefs->proxy.port = ini.GetInt("Port",1080,"Proxy");
	prefs->proxy.type = ini.GetInt("Type",PROXYTYPE_NOPROXY,"Proxy");
	SetLanguage();

	// Web Server
	m_sWebPassword.Put(ini.GetString("Password", "mypass", "WebServer").GetBuffer(0));
	m_nWebPort.Put(ini.GetInt("Port", 4711, "WebServer"));
	m_bWebEnabled.Put(ini.GetBool("Enabled", false, "WebServer"));
	m_bWebUseGzip.Put(ini.GetBool("UseGzip", true, "WebServer"));
	m_nWebPageRefresh.Put(ini.GetInt("PageRefreshTime", 120, "WebServer"));

	//////////////////////////////////////////////////////////////////
	// LANCAST (moosetea) - Read the settings

	// Get the Defualt Local IP address, using this clients hostname,
	// we will use this ip as the "default" Lancast ip, if no ip is found in the ini file.
	char localHost[255];
	gethostname(localHost, 255);						// Get this clients hostname
	struct hostent *host = gethostbyname(localHost);	// Get the host info

	// Get this clients default ip as a string
	char t_strdefaultip[17];
	strcpy(t_strdefaultip,inet_ntoa (*(struct in_addr*)*host->h_addr_list));

	// Get the ini file settings
	m_LancastEnabled.Put(ini.GetBool("Enabled",false,"Lancast"));
	m_UseDefaultIP.Put(ini.GetBool("UseDefaultIP",true,"Lancast"));
	m_DefaultLancastIP.Put(inet_addr(ini.GetString("IPAddress",t_strdefaultip,"Lancast")));
	m_LancastIP.Put(inet_addr(ini.GetString("IPAddress",t_strdefaultip,"Lancast")));
	m_LancastSubnet.Put(inet_addr(ini.GetString("Subnet","255.255.0.0","Lancast")));
	m_LancastPort.Put(ini.GetInt("Port",5000,"Lancast"));

	/////////////////////////////////////////////////////////////////////////////

	//START enkeyDEV(kei-kun) -Messaging preferences-
	m_nAcceptMessagesFrom.Put(ini.GetInt("AcceptMessageFrom",1));
	m_nAwayState.Put(false);
	m_sAwayStateMessage.Put(ini.GetString("AwayStateMessage","").GetBuffer(0));
	//END enkeyDEV(kei-kun) -Messaging preferences-
}

// LANCAST (moosetea) - get the lancast ip address
uint32 CPreferences::GetLancastIP(){
	if (GetLancastUseDefaultIP())
		return m_DefaultLancastIP.Get();
	else
		return m_LancastIP.Get();
}

WORD CPreferences::GetWindowsVersion(){
	if(!bWinVerAlreadyDetected)
	{	
		bWinVerAlreadyDetected = true;
		BOOL bIsNTNBased	=	FALSE;
		m_wWinVer = DetectWinVersion(bIsNTNBased);	
		m_bIsNTBased.Put(bIsNTNBased);
	}	
	return m_wWinVer;
}

uint16 CPreferences::GetDefaultMaxConperFive(){
	switch (GetWindowsVersion()){
		case _WINVER_98_: /*= _WINVER_NT4_*/
			if ( m_bIsNTBased.Get() )
			{
				return 10;
			}
			return 5;
		case _WINVER_95_:	
			return 5;
		case _WINVER_ME_:
			return 10;
		case _WINVER_2K_:
			return 20;
		case _WINVER_XP_:
			return 20;
		default:
			return 10;
	}
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
int CPreferences::GetColumnSortItem(Table t) const
{
	switch(t) 
	{
		case tableDownload:
			return prefs->tableSortItemDownload;
		case tableUpload:
			return prefs->tableSortItemUpload;
		case tableQueue:
			return prefs->tableSortItemQueue;
		case tableSearch:
			return prefs->tableSortItemSearch;
		case tableShared:
			return prefs->tableSortItemShared;
		case tableServer:
			return prefs->tableSortItemServer;
	}
	return 0;
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
bool CPreferences::GetColumnSortAscending(Table t) const
{
	switch(t) 
	{
		case tableDownload:
			return prefs->tableSortAscendingDownload;
		case tableUpload:
			return prefs->tableSortAscendingUpload;
		case tableQueue:
			return prefs->tableSortAscendingQueue;
		case tableSearch:
			return prefs->tableSortAscendingSearch;
		case tableShared:
			return prefs->tableSortAscendingShared;
		case tableServer:
			return prefs->tableSortAscendingServer;
	}
	return true;
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortItem(Table t, int sortItem)
{
	switch(t) 
	{
		case tableDownload:
			prefs->tableSortItemDownload = sortItem;
			break;
		case tableUpload:
			prefs->tableSortItemUpload = sortItem;
			break;
		case tableQueue:
			prefs->tableSortItemQueue = sortItem;
			break;
		case tableSearch:
			prefs->tableSortItemSearch = sortItem;
			break;
		case tableShared:
			prefs->tableSortItemShared = sortItem;
			break;
		case tableServer:
			prefs->tableSortItemServer = sortItem;
			break;
	}
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortAscending(Table t, bool sortAscending)
{
	switch(t) 
	{
		case tableDownload:
			prefs->tableSortAscendingDownload = sortAscending;
			break;
		case tableUpload:
			prefs->tableSortAscendingUpload = sortAscending;
			break;
		case tableQueue:
			prefs->tableSortAscendingQueue = sortAscending;
			break;
		case tableSearch:
			prefs->tableSortAscendingSearch = sortAscending;
			break;
		case tableShared:
			prefs->tableSortAscendingShared = sortAscending;
			break;
		case tableServer:
			prefs->tableSortAscendingServer = sortAscending;
			break;
	}
}

void CPreferences::SetLanguage() {
	switch(m_nLanguageID.Get()) {
		// read from ini and supported by the program?
		case MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "english");break;
		case MAKELANGID(LANG_GERMAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "german");break;
		case MAKELANGID(LANG_FRENCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "french");break;
		case MAKELANGID(LANG_DANISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "danish");break;
		case MAKELANGID(LANG_SPANISH,SUBLANG_SPANISH):setlocale( LC_ALL, "spanish");break;
		case MAKELANGID(LANG_DUTCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "dutch");break;
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "russian");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):setlocale( LC_ALL, "chs");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):setlocale( LC_ALL, "cht");break;
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "korean");break;
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "polish");break;
		case MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE_BRAZILIAN):setlocale( LC_ALL, "ptb");break;
		case MAKELANGID(LANG_FINNISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "finnish");break;
		case MAKELANGID(LANG_ITALIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "italian");break;
		case MAKELANGID(LANG_CATALAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "esp");break;
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):setlocale( LC_ALL, "greek");break;
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):setlocale( LC_ALL, "hebrew");break;
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "lithuanian");break;
		case MAKELANGID(LANG_ROMANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "romanian");break;
		default:{
			uint8 nLocalLang = ::GetThreadLocale();
			
            CString strLng1 = GetResString(IDS_MB_LANGUAGEINFO,MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
			CString strLng2 = "";
			switch(nLocalLang) {
				case	LANG_ENGLISH:
				case	LANG_GREEK:
				case 	LANG_GERMAN:
				case 	LANG_FRENCH:
				case 	LANG_DANISH:
				case 	LANG_SPANISH:
				case 	LANG_DUTCH:
				case 	LANG_RUSSIAN:
				case 	LANG_CHINESE:
				case 	LANG_KOREAN:
				case 	LANG_POLISH:
				case 	LANG_PORTUGUESE:
				case	LANG_FINNISH:
				case	LANG_ITALIAN:
				case	LANG_CATALAN:
				case	LANG_HEBREW:
				case	LANG_LITHUANIAN:
				case    LANG_ROMANIAN:
				{
					m_nLanguageID.Put(MAKELANGID(nLocalLang,SUBLANG_DEFAULT));
					//strLng2 = GetResString(IDS_MB_LANGUAGEINFO,prefs->languageID);
					strLng1 = GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID.Get());//Cax2 fix
					break;
				}
				default:
					m_nLanguageID.Put(MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
			}
			if ( m_bIsNTBased.Get() )
			{
				SetThreadLocale(m_nLanguageID.Get()); // THIS FUNCTION DOEN NOT WORK ON WIN9X !!!!!!!!1
			}
			else
			{
				ASSERT(false); // locale NOT set because we just can't do it on Win9x!!!!
			}

			CString strMessage = strLng1 + CString("\n\n") + strLng2;;
			MessageBox(0, strMessage.GetBuffer(),"Language",64);
			
			// DonGato
			CString strQuestion;
			if (nLocalLang == LANG_SPANISH)
				strQuestion = CString("\xbf") + GetResString(IDS_MULTIPLE, m_nLanguageID.Get()) + CString("?");
			else
				strQuestion = GetResString(IDS_MULTIPLE, m_nLanguageID.Get()) + CString("?");
			
			if (IDNO==MessageBox(0, strQuestion.GetBuffer(),"eMule Plus",MB_YESNO))
				return;
			else
				m_bMultiple.Put(true);
		}
	}
	if ( m_bIsNTBased.Get() )
	{
		SetThreadLocale(m_nLanguageID.Get());
	}

	// if loading a string fails, set language to English
	if (GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID.Get()).GetLength()==0) {
		m_nLanguageID.Put(MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
		if ( theApp.glob_prefs->IsNTBased() )
		{
			SetThreadLocale(m_nLanguageID.Get());
		}
	}
}

void CPreferences::SetMainProcessPriority(int Value) 			
{
	m_nMainProcessPriority.Put(Value);
}

BOOL CPreferences::IsNTBased(void)
{
	GetWindowsVersion();
	return (m_bIsNTBased.Get());
}
